#include "core/core.h"
#include <stdlib.h>
#include <stdio.h>
#include <dir.h>
#include <math.h>
#include <time.h>
// for LOCATE, COLOR, INKEY and INSTAT
#include <conio.h>
// for threads
#include <process.h>
#include <windows.h>

#undef RGB
#undef EOF

namespace bpp
{

#include "System.h"

namespace System
{

static char copyright[] =
	"B++ run-time library (c) 2001, 2002 Pavel \"EvilOne\" Minayev";
array<string> Command;


double __stdcall Abs(double n)
{
	return n < 0 ? -n : n;
}


string __stdcall ANSIToOem(string s)
{
	string result;
	result.alloc(s.length());
	CharToOemBuff(s.cstr(), result.cstr(), s.length());
	return result;
}


unsigned char __stdcall Asc(string s)
{
	return s.cstr()[0];
}


double __stdcall Ceil(double n)
{
	return ceil(n);
}


void __stdcall ChDir(string dir)	// not ANSI C++
{
	chdir(dir.cstr());
}


string __stdcall Chr(unsigned char n)
{
	char tmp[2] = { n, 0 };
	return tmp;
}


int __stdcall CLng(double n)
{
	return n;
}


void __stdcall Cls()
{
	clrscr();
}


void __stdcall Color(int fg, int bg)
{
	textcolor(fg);
	if (bg >= 0)
		textbackground(bg);
}

double __stdcall Cos(double n)
{
	return cos(n);
}


ref<object> __stdcall CreateObject(string classname)
{
	rtti* cl = classlist::find(classname);
	if (cl)
		return cl->create();
	else
		return 0;
}


string __stdcall CrLf()
{
	return "\r\n";
}


void __stdcall Cursor(bool visible)
{
	_setcursortype(visible ? _NORMALCURSOR : _NOCURSOR);
}


string __stdcall Date()
{
	char buf[9];
	time_t tmr = time(NULL);
	tm* dt = localtime(&tmr);
	sprintf(buf, "%02d/%02d/%02d", ++dt->tm_mon, dt->tm_mday, dt->tm_year % 100);
	return buf;
}


string __stdcall Delete(string s, int start, int length)
{
	return Left(s, start - 1) + Mid(s, start + length + 1, 0x7FFFFFFF);
}


string __stdcall Dir(string mask)		// not ANSI C++
{
	static bool first = true;
	static ffblk fb;
	int err;
	if (Len(mask))
	{
		first ? first = false : findclose(&fb);
		err = findfirst(mask.cstr(), &fb, FA_RDONLY | FA_HIDDEN |
			FA_SYSTEM | FA_DIREC | FA_ARCH);
	}
	else
		err = findnext(&fb);
	return err ? "" : fb.ff_name;
}


bool __stdcall DirExists(string dir)
{
	return FileExists(dir + "\\nul");
}


string __stdcall Environ(string var)
{
	return getenv(var.cstr());
}


double __stdcall Exp(double n)
{
	return exp(n);
}


bool __stdcall ExtractResource(string name, variant vtype, string filename)
{
	HINSTANCE hInst = GetModuleHandle(NULL);
	HRSRC hResInfo;
	HGLOBAL hResource;
	HANDLE file;
	DWORD writen;
	string type;
	if (vtype.vartype == variant::num)
		type = (char*)(int)vtype;
	else
		type = vtype;
	if (!(hResInfo = FindResource(hInst, name.cstr(), type.cstr())))
		return false;
	if (!(hResource = LoadResource(hInst, hResInfo)))
		return false;
	file = CreateFile(filename.cstr(), GENERIC_WRITE | GENERIC_READ,
		0, 0, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
	if (file == INVALID_HANDLE_VALUE)
		return false;
	WriteFile(file, (UCHAR*)LockResource(hResource),
		SizeofResource(hInst, hResInfo), &writen, NULL);
	CloseHandle(file);
	return true;
}


bool __stdcall FileExists(string filename)
{
	FILE* f = fopen(filename.cstr(), "rb");
	if (f)
		fclose(f);
	return f;
}


double __stdcall Fix(double n)
{
	return int(n);
}


double __stdcall Floor(double n)
{
	return floor(n);
}


double __stdcall Frac(double n)
{
	return n - Fix(n);
}


string __stdcall FullDate()
{
	char buf[9];
	time_t tmr = time(NULL);
	tm* dt = localtime(&tmr);
	sprintf(buf, "%02d/%02d/%04d", ++dt->tm_mon, dt->tm_mday, 1900 + dt->tm_year);
	return buf;
}


string __stdcall Hex(unsigned long n)
{
	char buf[32];
	sprintf(buf, "%X", n);
	return buf;
}


variant __stdcall IIf(bool cond, variant ontrue, variant onfalse)
{
	return cond ? ontrue : onfalse;
}


string __stdcall InKey()
{
	if (!kbhit())
		return "";
	char result[] = { 0, 0, 0 };
	if (!(result[0] = getch()))
	{
		result[0] = 27;
		result[1] = getch();
	}
	return result;
}


string __stdcall Input_STRING(int count)
{
	string result;
	while (count--)
	{
		while (!InStat());
		string key = InKey();
		if (key.length() == 1)
			result = result + key;
	}
	return result;
}


string __stdcall Insert(string s, string sub, int pos)
{
	return Left(s, pos) + sub + Mid(s, pos + 1, 2000000000);
}


bool __stdcall InStat()
{
	return kbhit();
}


int __stdcall InStr(variant vstart, string s, string sub)
{
	int start;
	if (!sub.length())
	{
		sub = s;
		s = vstart;
		start = 0;
	}
	else
	{
		start = vstart;
		start--;
		if (start < 0)
			start = 0;
		else if (start > (s.length() - sub.length()))
			return 0;
	}
	int n = s.length() - sub.length() + 1;
	char* p = s.cstr() + start;
	for (int i = start; i < n; i++, p++)
		if (!memcmp(p, sub.cstr(), sub.length()))
			return i + 1;
	return 0;	
}


int __stdcall InStrRev(string s, string sub, int start)
{
	start--;
	if (start >= 0)
	{
		if (start > (s.length() - sub.length()))
			start = s.length() - sub.length();
		else if (start < 0)
			return 0;
	}
	else
		start = s.length() - sub.length();
	char* p = s.cstr() + start;
	for (int i = start; i >= 0; i--, p--)
		if (!memcmp(p, sub.cstr(), sub.length()))
			return i + 1;
	return 0;	
}


int __stdcall Int(double n)
{
	return n;
}


void __stdcall Kill(string filename)
{
	remove(filename.cstr());
}


string __stdcall LCase(string s)
{
	string result = s + "";
	CharLowerBuff(result.cstr(), result.length());
	return result;	
}


string __stdcall Left(string s, int count)
{
	string result;
	if (count < 0)
		count = 0;
	else if (count > s.length())
		count = s.length();
	result.alloc(count);
	memcpy(result.cstr(), s.cstr(), count);
	return result;
}


int __stdcall Len(string s)
{
	return s.length();
}


void __stdcall Locate(int x, int y)
{
	gotoxy(x, y);
}


double __stdcall Log(double n)
{
	return log(n);
}


string __stdcall LTrim(string s)
{
	char* p = s.cstr();
	while (*p == 32)
		p++;
	return p;
}


string __stdcall MakeIntResource(unsigned short id)
{
	return (char*)id;
}


int __stdcall MemCmp(int p1, int p2, int size)
{
	return memcmp((void*)p1, (void*)p2, size);
}


void __stdcall MemCpy(int p1, int p2, int count)
{
	memcpy((void*)p1, (void*)p2, count);
}


void __stdcall MemSet(int dest, unsigned char ch, int count)
{
	memset((void*)dest, ch, count);
}


string __stdcall Mid(string s, int start, int count)
{
	int len = Len(s);
	if (count < 0)
		count = 0;
	if (start < 1 || start > len)
		return "";
	if (start + count - 1 > len)
		count = len - start + 1;
	string result;
	result.alloc(count);
	char* p = result.cstr(), *q = s.cstr() + start - 1;
	memcpy(result.cstr(), s.cstr() + start - 1, count);
	return result;
}


void __stdcall MkDir(string dir)	// not ANSI C++
{
	mkdir(dir.cstr());
}


string __stdcall NullStr()
{
	return string((char*)0);
}


string __stdcall OemToANSI(string s)
{
	string result;
	result.alloc(s.length());
	OemToCharBuff(s.cstr(), result.cstr(), s.length());
	return result;
}


string __stdcall PadC(string s, int len, string c)
{
	int l = Len(s);
	if (len < l)
		return s;
	string pad = String((len - l) / 2, c);
	string result = pad + s + pad;
	if ((len - l) % 2)
		result += c;
	return result;
}


string __stdcall PadL(string s, int len, string c)
{
	int l = Len(s);
	if (len < l)
		return s;
	else
		return String(len - l, c) + s;
}


string __stdcall PadR(string s, int len, string c)
{
	int l = Len(s);
	if (len < l)
		return s;
	else
		return s + String(len - l, c);
}


double __stdcall Pow(double n, double p)
{
	return pow(n, p);
}


void __stdcall Randomize(int seed)
{
	srand(seed);
}


void __stdcall Rename(string oldname, string newname)
{
	rename(oldname.cstr(), newname.cstr());
}


string __stdcall Replace(string s, string sub, int pos)
{
	return Left(s, pos - 1) + sub + Mid(s, pos + Len(sub) + 1, 0x7FFFFFFF);
}


string __stdcall Reverse(string s)
{
	string result;
	result.alloc(s.length());
	char* p = result.cstr();
	char* q = s.cstr() + s.length() - 1;	
	for (int i = 0; i < s.length(); i++, p++, q--)
		*p = *q;
	return result;
}


int __stdcall RGB(unsigned char r, unsigned char g, unsigned char b)
{
	return b + (g << 8) + (r << 16);
}


string __stdcall Right(string s, int count)
{
    int len = Len(s);
	string result;
	if (count < 0)
		count = 0;
	else if (count > len)
		count = len;
	result.alloc(count);
	memcpy(result.cstr(), s.cstr() + len - count, count);
	return result;
}


int __stdcall RInStr(variant vstart, string s, string sub)
{
	if (!sub.length())
		return InStrRev(vstart, s, -1);
	else
		return InStrRev(s, sub, vstart);
}


void __stdcall RmDir(string dir)	// not ANSI C++
{
	rmdir(dir.cstr());
}


double __stdcall Rnd(int max)
{
	if (max)
		return rand() * max / RAND_MAX;
	else
		return double(rand()) / RAND_MAX;
}


int __stdcall Round(double n)
{
	if (Frac(n) < 0.5)
		return Floor(n);
	else
		return Ceil(n);
}


string __stdcall RTrim(string s)
{
	char* p = s.cstr() + Len(s);
	do
	{
		p--;
		if (p < s.cstr())
			return s;
	} while (*p == 32);
	return Left(s, p - s.cstr() + 1);
}


int __stdcall Sgn(double n)
{
	return n ? n / abs(n) : 0;
}


void __stdcall Shell(string cmd)
{
	system(cmd.cstr());
}


double __stdcall Sin(double n)
{
	return sin(n);
}


void __stdcall SleeP(double sec)
{
	if (sec < 0)
	{
		while (!InStat());
		while (InStat()) InKey();
	}
	else
		Sleep(sec * 1000);
}


string __stdcall Space(int count)
{
	string result;
	result.alloc(count);
	memset(result.cstr(), 32, count);
	return result;
}


double __stdcall Sqr(double n)
{
	return sqrt(n);
}


string __stdcall Str(double n)
{
	char tmp[256];
	sprintf(tmp, "%.16g", n);
	return tmp;
}


string __stdcall Str0(double n, int len, int frac)
{
	char tmp[256];
	string fmt = string("%0") + Str(len) + string(".") + Str(frac) + "g";
	sprintf(tmp, fmt.cstr(), n);
	return tmp;
}


string __stdcall String(int count, variant s)
{
	string c = (s.vartype == variant::num ? Chr(s) : string(s));
	int len = Len(c);
	char *buf = new char[len * count + 1], *p = buf;
	for (int i = 0; i < count; i++, p += len)
		memcpy(p, c.cstr(), len);
	*p = 0;
	string result = buf;
	delete buf;
	return result;
}


int __stdcall Tally(string s, string sub)
{
	int result = 0;
	int n = s.length() - sub.length() + 1;
	char* p = s.cstr();
	while (n--)
		if (!memcmp(p++, sub.cstr(), sub.length()))
			result++;
	return result;
}


double __stdcall Tan(double n)
{
	return tan(n);
}


string __stdcall Time()
{
	char buf[9];
	time_t tmr = time(NULL);
	tm* dt = localtime(&tmr);
	sprintf(buf, "%02d:%02d:%02d", dt->tm_hour, dt->tm_min, dt->tm_sec);
	return buf;
}


double __stdcall Timer()
{
	return (double)GetTickCount() / 1000;
}


string __stdcall Trim(string s)
{
	return LTrim(RTrim(s));
}


string __stdcall UCase(string s)
{
	string result = s + "";
	CharUpperBuff(result.cstr(), result.length());
	return result;	
}


double __stdcall Val(string s)
{
	return variant(s);
}


int __stdcall VarPtr(void* var)
{
	return (int)var;
}


string __stdcall VarPtr_STRING(int ptr)
{
	return string((char*)ptr);
}


int __stdcall VarType(variant& v)
{
	return v.vartype;
}


void __stdcall Width(int w, int h)
{
	if (h < 0)
	{
		text_info ti;
		gettextinfo(&ti);
		h = ti.screenheight;
	}
	if (w == 40 && h == 25)
		textmode(C40);
	else if (w == 80 && h == 25)
		textmode(C80);
	else if (w == 80 && (h == 43 || h == 50))
		textmode(C4350);
}



void __stdcall Window(int x1, int y1, int x2, int y2)
{
	window(x1, y1, x2, y2);
}


int Stream::Position()
{
	return Seek(0, 1);
}


bool Stream::EOF()
{
	return Position() >= Size();
}


void Stream::Position(int pos)
{
	Seek(pos, 0);
}


void Stream::Read(void* p, int size)
{
	ReadBlock(p, size);
}


void Stream::Read(string& s)
{
	int len;
	Read(len);
	s.alloc(len);
	ReadBlock(&s, len);
}


void Stream::Read(apistring& s)
{
	int len;
	Read(len);
	s.alloc(len);
	ReadBlock(&s, len);
}


void Stream::Read(ref<object>& obj)
{
	obj->LoadFromStream(this);
}


int Stream::ReadBlock(void* block, int size)
{
	return 0;
}


string Stream::ReadLine()
{
	char result[1024];
	int i;
	for (i = 0; i < 1023; i++)
	{
		char c;
		c = ReadStr(1).cstr()[0];
		if (c == 10)
			break;
		else if (c == 13)
			i--;
		else
			result[i] = c;
	}
	result[i] = 0;
	return result;
}


ref<object> Stream::ReadObject()
{
	string classname;
	Read(classname);
	ref<object> obj = CreateObject(classname);
	if (obj.ptr)
		obj->LoadFromStream(this);
	return obj;
}


string Stream::ReadStr(int len)
{
	string result;
	result.alloc(len);
	ReadBlock(&result, len);
	return result;	
}


int Stream::Seek(int offset, int base)
{
	return -1;
}


int Stream::Size()
{
	int pos = Position(), size = Seek(0, 2);
	Position(pos);
	return size;
}


void Stream::Write(void* p, int size)
{
	WriteBlock(p, size);
}


void Stream::Write(string& s)
{
	int len = s.length();
	Write(len);
	WriteBlock(&s, len);
}


void Stream::Write(apistring& s)
{
	int len = s.length();
	Write(len);
	WriteBlock(&s, len);
}


void Stream::Write(ref<object>& obj)
{
	obj->SaveToStream(this);
}


int Stream::WriteBlock(void* block, int size)
{
	return 0;
}


void Stream::WriteObject(ref<object> obj)
{
	string classname = obj->ClassName();
	Write(classname);
	obj->SaveToStream(this);
}


void Stream::WriteStr(string s)
{
	WriteBlock(&s, Len(s));
}


void Stream::WriteLine(string line)
{
	WriteBlock(&line, Len(line));
	WriteStr("\r\n");
}


bool File::Open(string filename, int mode)
{
	string fm;
	switch (mode)
	{
	case fmOpenRead:		fm = "rb"; break;
	case fmOpenReadWrite:	fm = "r+b"; break;
	case fmCreateWrite:		fm = "wb"; break;
	case fmCreateReadWrite:	fm = "w+b"; break;
	}
	return CID = fopen(filename.cstr(), fm.cstr());
}


void File::Close()
{
	if (CID)
	{
		fclose((FILE*)CID);
		CID = 0;
	}
}


int File::ReadBlock(void* block, int size)
{
	return fread(block, size, 1, (FILE*)CID);
}


int File::Seek(int offset, int base)
{
	fseek((FILE*)CID, offset, base);
	return ftell((FILE*)CID);
}


int File::WriteBlock(void* block, int size)
{
	return fwrite(block, size, 1, (FILE*)CID);
}


void File::Initialize()
{
	CID = 0;
}


void File::Terminate()
{
	Close();
}


void Thread::Terminate()
{
	Stop();
}


void Thread::Run()
{
	if (mHandle)
		return;
	union
	{
		void (Thread::*main)();
		void (*thread)(void*);
	}; main = &Thread::Stub;
	mHandle = _beginthread(thread, mStackSize, this);
	GenericEvent *evt = new GenericEvent;
	evt->Sender = this;
	OnRun.raise(bpp::ref<GenericEvent>(evt));
}


void Thread::Stop()
{
	TerminateThread((HANDLE)mHandle, 0);
}


void Thread::Stub()
{
	// perform exception handling here and terminate
	// the entire program if necessary
	try
	{
		try
		{
			Main();
		}
		__except (process_se(GetExceptionCode()))
		{
		}
	}
	catch (const string& s)
	{
		abort_with_error(s);
	}
}


int Thread::Handle()
{
	return mHandle;
}


int Thread::StackSize()
{
	return mStackSize;
}


bool Thread::Running()
{
	DWORD result;
	GetExitCodeThread((HANDLE)mHandle, &result);
	return result == STILL_ACTIVE;
}


void Thread::StackSize(int value)
{
	mStackSize = value;
}


void Thread::Suspended(bool value)
{
	if (value)
		SuspendThread((HANDLE)mHandle);
	else
		ResumeThread((HANDLE)mHandle);
}


void Thread::Main()
{
}


void Mutex::Initialize()
{
	mHandle = (int)CreateMutex(NULL, FALSE, NULL);
}


void Mutex::Terminate()
{
	CloseHandle((HANDLE)mHandle);
}


bool Mutex::Capture(int timeout)
{
	if (!timeout)
		timeout = INFINITE;
	DWORD result = WaitForSingleObject((HANDLE)mHandle, timeout);
	return result != WAIT_FAILED && result != WAIT_TIMEOUT;
}


void Mutex::Release()
{
	ReleaseMutex((HANDLE)mHandle);
}


int Mutex::Handle()
{
	return mHandle;
}


bool Mutex::Captured()
{
	DWORD result = WaitForSingleObject((HANDLE)mHandle, 0);
	return result != WAIT_FAILED && result != WAIT_TIMEOUT;
}


initialization::initialization()
{
	char* cmd = GetCommandLine();
	string args[255];
	int i = 0;
	while (i < 256)
	{
		while (*cmd == ' ' || *cmd == '\t')
			cmd++;
		if (!*cmd)
			break;
		char arg[1024], *p = arg;
		if (*cmd == '"')
		{
			cmd++;
			while (*cmd && *cmd != '"')
				*(p++) = *(cmd++);
			if (*cmd)
				cmd++;							
		}
		else
		{
			while (*cmd && *cmd != ' ' && *cmd != '\t')
				*(p++) = *(cmd++);
		}
		*p = 0;
		args[i++] = string(arg);
	}
	i--;
	Command.redim(0, i);
	for (int j = 0; j <= i; j++)
	{
		Command(j) = args[j];
	}
}


finalization::~finalization()
{
	copyright[0] = 0;
}


}
}