#include "bpp.h"
#include "parser.h"
#include "util.h"
#include <string>
#include <iostream>
#include <time.h>

using namespace std;


int main(int argc, char* argv[])
{
	cout << "\nB++ compiler version " << version.major << '.'
		<< version.minor << '.' << version.release << '\n';
	cout << "Copyright (c) 2001, 2002 Pavel \"EvilOne\" Minayev\n";
	if (argc < 2)
	{
		cout <<
			"Usage: bpp [<options>] <module-name>\n"
			"  -l  Compile library          -p  Dump C++ output only\n"
			"  -h  Generate C++ header      -r  Don't produce debugging info\n";
		return 1;
	}
	bool lib = false, hdr = false, devmode = true;
	bool verbose = false, process = false;
	int i;
	for (i = 1; i < argc; i++)
	{
		if (argv[i][1] && (argv[i][0] == '-' || argv[i][0] == '/'))
		{
			switch (argv[i][1])
			{
			case 'p': process = true; break;
			case 'l': lib = true; break;
			case 'h': hdr = true; break;
			case 'r': devmode = false; break;
			}
		}
		else
			break;
	}
	try
	{
		int timer = clock();
		cout << "Compiling module: " << argv[i] << "\n";
		cout << "   Processing...\n";
		parser parser(argv[i], getexename() + "\\..\\..\\");
		parser.parse(lib, devmode);
		if (hdr)
			parser.write();
		if (!process)
		{
			cout << "   Compiling...\n";
			parser.compile(verbose);
		}
		cout << "Done in " << float(clock() - timer) / CLK_TCK
			<< " seconds\n";
	}
	catch (parse_error err)
	{
		cout << "Error in '" << err.module << "' at "
			<< err.line << "," << err.col << ": "
			<< err.error << "\n";
	}
	catch (string msg)
	{
		cout << msg << "\n";
	}
	catch (char* msg)
	{
		cout << msg << "\n";
	}
	return 0;
}