#ifndef _IDENT_H
#define _IDENT_H

#include <stdio.h>
#include <string>
#include <iostream>
#include "table.h"

using namespace std;

class module;


class ident
{
public:
	enum kinds { none, cons, type, var, func, struc, cls, als };
	enum scopes { local, priv, prot, pub, pbsh };

	module* owner;
	int kind, scope;
	string name;
	
	ident(FILE* in, module* mod);
	ident(int kind, const string& name, int scope);
	string deftype() const;
	static ident* load(FILE* in, module* mod);
	virtual string cname(bool full = true) const;
	virtual void save(FILE* out) const;
	virtual void write(ostream& out) const;
	virtual void* operator() () { return this; }
};


class alias: public ident
{
public:
	ident* aliased;
	bool type;

	alias(FILE* in, module* mod);
	alias(const string& name, ident* aliased, bool type = false):
		aliased(aliased), ident(als, name, aliased->scope), type(type) { }
	virtual void save(FILE* out) const;
	virtual void* operator() () { return aliased; }
};


class constant: public ident
{
public:
	string value;
	bool str;

	constant(FILE* in, module* mod);
	constant(const string& name, int scope);
	virtual void save(FILE* out) const;
	virtual void write(ostream& out) const;
};


class udt: public ident
{
public:
	udt(FILE* in, module* mod);
	udt(const string& name);
	udt(int kind, const string& name, int scope);
	void put(FILE* out) const;
	virtual string cname(bool full = true) const;
};


class variable: public ident
{
public:
	enum coltypes { scal, coll, dict };

	udt* type;
	int level, statar, col, flstr, stdims[5];
	bool byref, array, propget, propset;
	string defval;

	variable(FILE* in, module* mod);
	variable(const string& name, int scope, int level = 0, udt* type = NULL);
	virtual void save(FILE* out) const;
	virtual void write(ostream& out) const;
};


class function: public ident
{
public:
	udt* type;
	table<variable*> args;
	bool method, resolved, external, generic;

	function(FILE* in, module* mod);
	function(const string& name, int scope, bool resolved, 
		bool method = false, udt* type = NULL);
	virtual void save(FILE* out) const;
	virtual void write(ostream& out) const;
};


class structure: public udt
{
public:
	table<variable*> fields;

	structure(FILE* in, module* mod);
	structure(const string& name, int scope);
	virtual void save(FILE* out) const;
	virtual void write(ostream& out) const;
};


class clas: public udt
{
public:
	clas* super;
	table<variable*> props;
	table<function*> methods;
	bool resolved;

	clas(FILE* in, module* mod);
	clas(const string& name, int scope, clas* super = NULL);
	variable* prop(const string& name);
	function* method(const string& name);
	ident* member(const string& name, bool inherited = true);
	bool childof(clas* c);
	virtual void save(FILE* out) const;
	virtual void write(ostream& out) const;
};



#endif