#include "lexer.h"


lexer::lexer(istream& in):
	in(in)
{
	c = in.get();
	line = 1;
}


bool lexer::isspace(char c) const
{
	return c == 9 || c == 32 || c == 13;
}


bool lexer::isalpha(char c) const
{
	return (c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z') || c == '_';
}


bool lexer::issuffix(char c) const
{
	switch (c)
	{
	case '!':	case '#':	case '$':
	case '%':	case '&':	case '@':
		return true;
	default:
		return false;	
	}
}


bool lexer::isdigit(char c) const
{
	return c >= '0' && c <= '9';
}


bool lexer::ishexdigit(char c) const
{
	return isdigit(c) || (c >= 'A' && c <= 'F') || (c >= 'a' && c <= 'f');
}


bool lexer::issymbol(char c) const
{
	switch (c)
	{
	case '+':	case '-':	case '*':
	case '/':	case '(':	case ')':
	case '&':	case '.':	case ',':
	case '=':	case '<':	case '>':
	case ';':	case ':':	case '[':
	case ']':
		return true;
	default:
		return false;
	}
}


void lexer::next()
{
	static int col = 1;
	while (isspace(c))
		col++, c = in.get();
	if (in.eof())
	{
		tokkind = nl;
		return;
	}
	this->col = col;
	token = "";
	if (c == '\'')
	{
		do
			c = in.get();
		while (c != '\n' && !in.eof());
		c = in.get();
		line++; col = 1;
		tokkind = nl;
	}
	else if (isalpha(c))
	{
		do
		{
			token += c;
			c = in.get();
		} while (isalpha(c) || isdigit(c));
		if (issuffix(c))
		{
			token += c;
			c = in.get();
		}
		if (c == ':')
		{
			char c1 = in.get();
			if (c1 == 10 || c1 == 13)
			{
				c = c1;
				tokkind = lab;
				goto endtok;
			}
			else
				in.putback(c1);
		}
		if (token == "_")
		{
			next();
			if (tokkind != nl)
				throw "newline expected";
			next();
		}
		else
		{
			mangle(token);
			tokkind = id;
		}
	}
	else if (isdigit(c))
	{
		do
		{
			token += c;
			c = in.get();
		} while (isdigit(c));
		if (c == '.')
		{
			do
			{
				token += c;
				c = in.get();
			} while (isdigit(c));
		}
		if (c == '&')
			c = in.get();
		tokkind = num;
	}
	else if (c == '&')
	{
		c = in.get();
		if (c == 'h' || c == 'H')
		{
			unsigned int number = 0;
			c = in.get();
			if (!ishexdigit(c))
				throw "hexadecimal number expected";
			do
			{
				int digit;
				if (c >= 'A' && c <= 'F')
					digit = 10 + c - 65;
				else if (c >= 'a' && c <= 'f')
					digit = 10 + c - 97;
				else
					digit = c - 48;
				number = number * 16 + digit;
				c = in.get();
			} while (ishexdigit(c));
			if (c == '&')
				c = in.get();
			while (number)
			{
				token = char(48 + number % 10) + token;
				number /= 10;
			}
		}
		else
			throw "number base (H, O or B) expected";
		tokkind = num;
	}
	else if (c == '"')
	{
		do
		{
			token += c;
			if (c == '\\')
				token += c;
			c = in.get();
		} while (c != '"' && c != 13 && !in.eof());
		if (c != '"')
			throw "string constant exceeds line";
		token += c;
		c = in.get();
		tokkind = str;
	}
	else if (c == '+' || c == '-')
	{
		char prev = c;
		token += c;
		c = in.get();
		if (c == '=' || c == prev)
		{
			token += c;
			c = in.get();
		}
		tokkind = sym;
	}
	else if (c == '*')
	{
		token += c;
		c = in.get();
		if (c == '=')
		{
			token += c;
			c = in.get();
		}
	}
	else if (c == '/')
	{
		token += c;
		c = in.get();
		if (c == '*')
		{
			col += 2;
			char prev;
			do
			{
				if (c == 10)
					line++, col = 1;
				else if (c != 13)
					col++;
				prev = c;
				c = in.get();
			} while (!(c == '/' && prev == '*') && !in.eof());
			col++;
			c = in.get();
			next();
			return;
		}
		else if (c == '=')
		{
			token += c;
			c = in.get();
		}
	}
	else if (c == ';' || c == '(' || c == ')' || c == '.' ||
		c == ',' || c == '=' || c == '[' || c == ']')
	{
		token += c;
		c = in.get();
		tokkind = sym;
	}
	else if (c == '>')
	{
		token += c;
		c = in.get();
		if (c == '=')
		{
			token += c;
			c = in.get();
		}
	}
	else if (c == '<')
	{
		token += c;
		c = in.get();
		if (c == '=' || c == '>')
		{
			token += c;
			c = in.get();
		}
	}
	else if (c == '\n')
	{
		line++;	col = 1;
		c = in.get();
		tokkind = nl;
	}
	else if (c == ':')
	{
		c = in.get();
		tokkind = nl;
	}
	else
		throw "invalid character";
endtok:
	col += token.length();
}


void lexer::mangle(string &id)
{
	static const char* reserved[] = {
		"unsigned", "signed", "char", "short", "int", "long", "bool",
		"float", "double", "void", "auto", "static", "register",
		"const", "mutable", "volatile", "private", "public", "extern",
		"protected", "struct", "class", "union", "enum", "typedef", "typename",
		"if", "do", "while", "for", "switch", "case", "default",
		"break", "try", "catch", "throw", "new", "delete", "sizeof",
		"typeid", "type_info", "namespace", "using", "bpp", "is" };
	if (id[0] <= 'a' || id[0] >= 'z')
		return;
	for (int i = 0; i < (sizeof(reserved) / sizeof(char*)); i++)
		if (id == reserved[i])
		{
			id[0] -= 32;
			break;
		}
}
