#include "module.h"
#include "parser.h"
#include "bpp.h"
#include "util.h"
#include <windows.h>


const string signature = "BPM";


module::module(const string& name)
{
	this->name = name;
	init = final = false;
}


module::module(const string& name, parser& parser, bool lib)
{
	this->name = name;
	par = &parser;
	string filename = name + ".bpm", srcname = name + ".bpp";
	FILE *in = fopen(filename.c_str(), "rb");
	if (in)
	{
		HANDLE srch = CreateFile(srcname.c_str(), GENERIC_READ,
			FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, 0);
		if (srch != INVALID_HANDLE_VALUE)
		{
			HANDLE modh = CreateFile(filename.c_str(), GENERIC_READ,
				FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, 0);
			FILETIME modftime;
			FILETIME srcftime;
			GetFileTime(modh, NULL, NULL, &modftime);
			GetFileTime(srch, NULL, NULL, &srcftime);
			CloseHandle(modh);
			CloseHandle(srch);
			if (CompareFileTime(&modftime, &srcftime) == -1)
			{
				class parser par(name, parser.basepath);
				par.parse(true, parser.devmode);
				par.compile(false);
				if (!(in = fopen(filename.c_str(), "rb")))
					throw "module '" + name + "' not found";
				path = "";
			}
		}
		path = "";
	}
	else
	{
		if (fexist(srcname))
		{
			{
				class parser par(name, parser.basepath);
				par.parse(true);
				par.compile(false);
			}
			if (!(in = fopen(filename.c_str(), "rb")))
				throw "module '" + name + "' not found";
			path = "";
		}
		else
		{
			path = parser.basepath + "lib\\";
			filename = path + filename;
			if (!(in = fopen(filename.c_str(), "rb")))
				throw "module '" + filename + "' not found";
		}
	}
	if (freads(in) != signature)
		throw "invalid module format";
	fgetc(in); fgetc(in); fgetc(in);
	string mname = freads(in);
	while (mname != "")
	{
		try
		{
			parser.modules[mname];
		}
		catch (out_of_range)
		{
			parser.modules.push_back(new module(mname, parser, lib));
		}
		mname = freads(in);
	}
	init = fgetc(in);
	final = fgetc(in);
	{
		int typecnt = fgeti(in);
		types.reserve(typecnt);
		for (int i = 0; i < typecnt; i++)
			ident::load(in, this);
	}
	{
		int idcnt = fgeti(in);
		idents.reserve(idcnt);
		for (int i = 0; i < idcnt; i++)
			idents.push_back(ident::load(in, this));
	}
	parser.out << "\n}\n\n";
	write(parser);
	parser.out << "\n\nnamespace " << parser.me.name << "\n{\n";
/*
	if (!lib)
	{
		if (init)
			parser.out << "\n" << name << "::initialization __init_" << name << ";\n";
		if (final)
			parser.out << name << "::finalization __final_" << name << ";";
	}
*/
	parser.out << "\n";
}


void module::save(parser& parser)
{
	par = &parser;
	FILE *out = fopen((name + ".bpm").c_str(), "wb");
	fwrites(signature, out);
	fputc(version.major, out);
	fputc(version.minor, out);
	fputc(version.release, out);
	{
		table<module*>::iterator i;
		for (i = parser.modules.begin() + 1; i != parser.modules.end(); i++)
			fwrites((*i)->name, out);
		fputc(0, out);
	}
	fputc(init, out);
	fputc(final, out);
	{
		int count = 0;
		table<udt*>::iterator i;
		for (i = types.begin(); i != types.end(); i++)
			if ((*i)->scope == ident::pub)
				count++;
		fputi(count, out);
		for (i = types.begin(); i != types.end(); i++)
			if ((*i)->scope == ident::pub)
				(*i)->save(out);
	}
	{
		int count = 0;
		table<ident*>::iterator i;
		for (i = idents.begin(); i != idents.end(); i++)
			if ((*i)->scope == ident::pub)
				count++;
		fputi(count, out);
		for (i = idents.begin(); i != idents.end(); i++)
			if ((*i)->scope == ident::pub)
				(*i)->save(out);
	}
	fclose(out);
}


udt* module::gettype(FILE* in)
{
	string mname = freads(in), tname = freads(in);
	try
	{
		if (mname == "")
			return types[tname];
		else
			return par->modules[mname]->types[tname];
	}
	catch (out_of_range)
	{
		throw "error loading module - THIS SHOULD NEVER HAPPEN!\n"
			"\tmname = '" + mname + "'  tname = '" + tname + "'";
	}
}


void module::write(parser& par)
{
	ofstream& out = par.out;
	out << "namespace " << name << "\n{\n\n";
	if (init)
		out << "struct initialization { initialization(); };\n";
	if (final)
		out << "struct finalization { ~finalization(); };\n";
	{
		table<udt*>::iterator i;
		out << "\n";
		for (i = types.begin(); i != types.end(); i++)
			if ((*i)->scope == ident::pub)
				if ((*i)->kind == udt::cls)
					if (!((*i)->name ^ "OBJECT"))
						out << "class " << (*i)->cname(false) << ";\n";
		for (i = types.begin(); i != types.end(); i++)
			if ((*i)->scope == ident::pub)
				(*i)->write(out);
	}
	{
		table<ident*>::iterator i;
		for (i = idents.begin(); i != idents.end(); i++)
			if ((*i)->scope == ident::pub)
				(*i)->write(out);
	}
	out << "\n}";
}


void module::write(parser& par, string filename)
{
	if (!filename.size())
		filename = name + ".h";
	ofstream out(filename.c_str(), ios::out | ios::binary);
	out << "namespace " << name << "\n{\n\n";
	if (init)
		out << "struct initialization { initialization(); };\n";
	if (final)
		out << "struct finalization { ~finalization(); };\n";
	{
		table<udt*>::iterator i;
		out << "\n";
		for (i = types.begin(); i != types.end(); i++)
			if ((*i)->scope == ident::pub)
				if ((*i)->kind == udt::cls)
					if (!((*i)->name ^ "OBJECT"))
						out << "class " << (*i)->cname(false) << ";\n";
		for (i = types.begin(); i != types.end(); i++)
			if ((*i)->scope == ident::pub)
			{
				(*i)->owner = this;
				(*i)->write(out);
			}
	}
	{
		table<ident*>::iterator i;
		for (i = idents.begin(); i != idents.end(); i++)
			if ((*i)->scope == ident::pub)
				(*i)->write(out);
	}
	out << "\n}";
}