#ifndef _PARSER_H
#define _PARSER_H

#include <string>
#include <fstream>
#include <vector>
#include <map>
#include "lexer.h"
#include "module.h"
#include "table.h"

using namespace std;


class parser
{
public:
	friend class module;

	parser(string name, const string& basepath);
	~parser();
	void parse(bool lib, bool devmode = false);
	void write();
	void compile(bool verbose);

private:
	ifstream in;
	ofstream out;
	lexer lex;
	module me;
	string basepath, resfile;
	table<module*> modules;
	vector<string> trash, dlls;
	table<module*>::iterator locals;
	map<string, bool> labels;
	clas* inside;
	bool lib, dll, inter, infunc, explic, console, vbdll, apistr, devmode;
	int indent, level, loops;

	void registerbuiltins();
	string newline();
	udt* findtype(string udtname, module* mod = NULL);
	ident* findid(string idname, module* mod = NULL);
	void enter();
	void leave();
	bool reserved(const string& token);
	void validate(const string& id);
	void parse_module();
	udt* parse_funccall(function* func = NULL, bool lvalue = false);
	udt* parse_expr(bool lvalue = false, bool strict = false,
		bool array = false, bool col = false);
	udt* parse_expr(bool lvalue, bool strict, bool array, bool col, ostream* os);
	udt* parse_typename();
	void parse_builtin(int scope);
	void parse_dim(int scope);
	void parse_type(int scope);
	string parse_conexpr(bool& str);
	void parse_const(int scope);
	void parse_enum(int scope);
	void parse_func(int scope, bool proto = true, clas* cls = NULL);
	void parse_property(int scope, clas* cls = NULL);
	void parse_propgs(int scope);
	void parse_class(int scope);
	void parse_classdecl(int scope);
	void parse_block();
	bool parse_statement();
	void parse_setorcall();
	void parse_if();
	void parse_while();
	void parse_do();
	void parse_for();
	void parse_foreach();
	void parse_print();
	void parse_input();
	void parse_lineinput();
	void parse_redim();
	void parse_initialization();
	void parse_finalization();
	void parse_raise();
	void parse_try();
	void parse_throw();
	void parse_select();
	void parse_using();
	void parse_alias();
	void parse_exit();
	void parse_incr();
	void parse_decr();
	void parse_swap();
	void parse_option();
	void parse_label();
	void parse_goto();
	void write_dll_loader();
	void write_dll_unloader();
};


struct parse_error
{
	string error, module;
	int line, col;
};


#endif